/******************************************************************************/
/*
 * eRIC Example LowPowerMode V1.1
 * Low Power Radio Solutions
 * March 2016
 * Built with CCS V6.1.2.00015
 * This example will send/receive few bytes of data in Low power as per #defines below.
 * When WDT or TimerA0 is used, interrupt is triggered every 1sec and within which if Pin19 is held high for 1 sec then
 * selected LPM is exited and stored data is sent over air before entering LPM mode again.
 * Pin22 interrupt can also be used to exit LPM and send data and go back to LPM mode.
 * Data is received/sent through Uart when LPM0(1Mhz clock,7 Rxpowerlevel) and LPM1(32768 clock,7 Rx powerlevel) is used.
 * Since all clocks are OFF in LPM2(2uA) , data can be dealt when it exits LPM2
 */

/******************************************************************************/

//Version V1.1
#include<cc430f5137.h>
#include "eRIC.h"
#include <stdio.h>
#include <string.h>

volatile unsigned int Uart_buff_Timeout = 0; //V1.1

// Choose module type to ensure correct frequency selections
#define eRIC4
//#define	eRIC9

#define LPM_Transceiver          //Can receive and transmit data
//#define LPM_Transmitter        //Only transmits data

//#define LPM_Level0               //Draws ~120uA when used as transceiver and 98uA when used as only tranmitter.1048576 clock
#define LPM_Level1               //Draws ~37uA when used as transceiver and 6uA when used as only tranmitter.32768 clock
//#define LPM_Level2               //Draws only ~2uA

//#define LPM_Exit_WDT            //Can exit LPM  using WDT which is triggered every 1 sec and also Pin19 is needs to held high for 1 sec
#define LPM_Exit_Pin_Interrupt  //Can exit LPM  using Pin22 interrupt
//#define LPM_Exit_Timer          //Can exit LPM  using TimerA0 which is triggered every 1 sec and also Pin19 is needs to held high for 1 sec


volatile unsigned int TimeOut;
volatile const char TransmitData[] = {'L','P','M',' ','E','X','A','M','P','L','E'};
int main(void)
{
	eRIC_WDT_Stop();                   //Stop watch dog timer, just in case
	eRIC_GlobalInterruptDisable();     //Global interrupts disabled

	Pin19_SetAsInput();               //Pin19 is used to exit LPM in WDT/TimerA0 interrupt
	Pin19_PullDownEnable();
	Pin18_SetAsOutput();            //Indicator. Blinks every second when interrupt is triggered in WDT /TimerA0

	Pin4_FunctionUartARxD();
	Pin3_FunctionUartATxOUT();

#ifdef eRIC4
	eROS_Initialise(434000000);       //eROS initilaised at 434Mhz
#else
	eROS_Initialise(869750000);       //eROS initilaised at 869.75Mhz
#endif



#ifdef LPM_Level0
	eRIC_SetCpuFrequency(1048576);
	eRIC_UARTAInitialise(19200);
	TimeOut = 1095;                   //Uarttime out for receiving from PC
	eRIC_UartARxInteruptEnable();
	#ifdef LPM_Transceiver
	eRIC_RxPowerLevel = 7;               //Rxpower level as 7 which has 0.2% duty cycles, i.e 99.8% receiver off
	#else
	eRIC_RxPowerLevel = 8;              //Recevier is turned off.
	#endif
#elif defined LPM_Level1
	eRIC_SetCpuFrequency(32768);
	eRIC_UARTAInitialise(1200);       //Set 1200 baud rate, as the clock is at 32768.
	TimeOut = 555;                    //Uarttime out for receiving from PC
	eRIC_UartARxInteruptEnable();
	#ifdef LPM_Transceiver
	eRIC_RxPowerLevel = 7;               //Rxpower level as 7 which has 0.2% duty cycles, i.e 99.8% receiver off
	#else
	eRIC_RxPowerLevel = 8;              //Recevier is turned off.
	#endif
#else
	#ifndef LPM_Exit_Pin_Interrupt
	#define LPM_Exit_Pin_Interrupt            //AT LPM2, all clocks are off even WDT and Timer clocks are also off so only Pin interrupt works
	#endif
	eRIC_RxPowerLevel = 8;              //Recevier is turned off.
#endif

	eRIC_TxPowerLevel = 7;
	eRIC_RfBaudRate = 2400; // v1.1 THis is because at 32768 clock ,RF baud rate need to lower than 32768/9 = 3640 ~=2400(Nearest lower baud rate)

	eRIC_Power = 12;                   //Radio power at 12
	eRIC_RadioUpdate();



#ifdef LPM_Exit_WDT
	eRIC_WDT_Setup(eRICWDT_Cs_32k+eRICWDT_Interval_32768);    //Set WDT with 32k Clock and 32768 interval which will be 1 sec
	eRIC_WDT_InterruptEnable();                               //Enable WDT interrupt
	eRIC_WDT_Reset();
#elif defined LPM_Exit_Timer
	eRIC_TimerA0_Setup(eRICTimer_Cs_32k+eRICTimer_Div_1+eRICTimer_UpMode+eRICTimer_Reset);      //Set TimerA0 with 32k clock
	eRIC_TimerA0_Capture0orCompare0_Data = 32768; //1sec clock cycles then interrupt
	eRIC_TimerA0_Capture0orCompare0Setup(eRICTimer_CompareMode+eRICTimer_CCInterruptEnable+eRICTimer_OutputMode_OutputOnly);
#else //LPM_Exit_Pin_Interrupt
	Pin22_PullDownEnable();
	Pin22_InterruptEnable();
	Pin22_InterruptLow2High();
	Pin22_ClearInterruptFlag();
#endif


	eRIC_RfDataReceivedInterruptEnable();  //V1.1

	eRIC_GlobalInterruptEnable();
	while(1)       //Loop continously
	{
		char i =0 ;
		while(i<11)
		{
			eRIC_RadioTx_Buffer[i] = TransmitData[i];   //Data to be sent is "LPMEXAMPLE" when it exits LPM
			i++;
		}
		eRIC_RadioTx_BuffCount = 11;
		eRIC_RfSenddata();
		#ifdef LPM_Exit_WDT
		eRIC_WDT_Start();
		#endif

		#ifdef LPM_Level0
		eRIC_LPM_Level0();
		#elif defined LPM_Level1
		eRIC_LPM_Level1();
		#else //LPM_Level2
		eRIC_LPM_Level2();
		#endif
	}//end of while(1)
}//End of main



#ifdef LPM_Exit_WDT
#pragma vector= WDT_VECTOR
__interrupt void WDT_ISR(void)
{
	eRIC_WDT_Stop();
	Pin18_Toggle();
	if(Pin19_Read())
	{
		while(Pin19_Read());
		#ifdef LPM_Level0
		eRIC_LPM_ExitLevel0();
		#elif defined LPM_Level1
		eRIC_LPM_ExitLevel1();
		#else //LPM_Level2
		eRIC_LPM_ExitLevel2();
		#endif
	}
	eRIC_WDT_Start();
}
#endif

#ifdef LPM_Exit_Timer
#pragma vector=TIMER0_A0_VECTOR
__interrupt void TIMER0__A0_ISR(void) // set for every 1sec
{
	eRIC_TimerA0_Reset();
	Pin18_Toggle();
	if(Pin19_Read())
    {
		while(Pin19_Read());
		#ifdef LPM_Level0
		eRIC_LPM_ExitLevel0();
		#elif defined LPM_Level1
		eRIC_LPM_ExitLevel1();
		#else //LPM_Level2
		eRIC_LPM_ExitLevel2();
		#endif
    }
}
#endif

#ifdef LPM_Exit_Pin_Interrupt
#pragma vector= PORT2_VECTOR
__interrupt void PORT2_ISR(void)
{
	volatile unsigned int function =  P2IV;     // Read Interrupt Vector triggered
	switch(function)
	{
	    case 0x0C: //If interrupted on PORT 2.5
	    {
	    	Pin22_ClearInterruptFlag();
	    	Pin18_Toggle();
	    	while(Pin22_Read());
	    	#ifdef LPM_Level0
	    	eRIC_LPM_ExitLevel0();
	    	#elif defined LPM_Level1
	    	eRIC_LPM_ExitLevel1();
	    	#else //LPM_Level2
	    	eRIC_LPM_ExitLevel2();
	    	#endif
	    }
	    break;
	    default:
	    	break;
	}
}
#endif

#pragma vector=USCI_A0_VECTOR
__interrupt void USCI_A0_ISR(void)
{
	volatile unsigned int function = UCA0IV;             // Read Interrupt Vector triggered
	switch(function)
	{
	    case USCI_UCRXIFG:                   // Vector 2 - RXIFG UART RECEIVE
	    	eRIC_RadioTx_Buffer[eRIC_RadioTx_BuffCount++] = eRIC_UartAReceiveByte();
	    	Uart_buff_Timeout = TimeOut;
	    	while((Uart_buff_Timeout--))
	    	{
	    		do
	    	    {
	    			if(eRIC_UartARxHasInterrupted())//(UCA0IFG & UCRXIFG) // Has data started?
	    		    {
	    				eRIC_RadioTx_Buffer[eRIC_RadioTx_BuffCount++] = eRIC_UartAReceiveByte();;
	    				Uart_buff_Timeout = TimeOut;// Reset counter
	    			}
	    	    }while(eRIC_UartABusy());
	    	}
	    	eRIC_RfSenddata();
	    	break;
	    case USCI_UCTXIFG:                   // Vector 4 - TXIFG
	    	break;
	    default:
	    	break;
	}
}

//Added in V1.1
void eRIC_RfDataReceivedInterrupt()   //V1.1 Add code here to deal with  available received data..THis is triggered when interrupt is enabled and a packet is received
{
#ifndef LPM_Exit_WDT
	eRIC_WaitForStableClock();  //V1.1
#endif
	while(eRIC_Rxdata_available)
	{
		eRIC_UartASendByte(eRIC_ReadRfByte());
		while(eRIC_UartATxBufferIsBusy());
	}
	while(eRIC_UartABusy());  //Added V1.1
}


